# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# lib_zephir.py
#  
# librairie de fonctions pour le client Zephir
#       
###########################################################################

registered = 1
try:
    import zephir.zephir_conf.zephir_conf as config
except:
    registered = 0
import os,time,socket,locale
from zephir.eolerpclib import xmlrpclib
from creole.cfgparser import EoleDict

# définition de l'encodage du texte
# charset = locale.getpreferredencoding()
charset = 'UTF-8'
# répertoire publique d'uucp
public_dir = '/var/spool/uucppublic'
# définition du proxy XMLRPC
eole_dir = '/usr/share/eole'
zephir_dir = '/usr/share/zephir'

# recherche du répertoire du module zephir
import zephir
zephir_path = os.path.dirname(zephir.__file__)

# redéfinition de la classe pour supporter les timeouts
class TransportEole(xmlrpclib.SafeTransport):

#    def __init__(self, use_datetime = 0):
#        self.proxy = None
#        super(xmlrpclib.SafeTransport,self).__init__(use_datetime)
#
#    def set_proxy(self, proxy):
#        self.proxy = proxy
#
#    def make_connection(self, host):
#        if self.proxy is not None:
#            self.realhost = host
#            https = httplib.HTTPS(self.proxy)
#            return https
#        else:
#            return super(xmlrpclib.SafeTransport,self).make_connection()
#
#    def send_request(self, connection, handler, request_body):
#        connection.putrequest("POST", 'https://%s%s' % (self.realhost, handler))
#
#    def send_host(self, connection, host):
#        connection.putheader('Host', self.realhost)

    def _parse_response(self, file, sock):
        """lit et parse la réponse du serveur
        modifié pour ne pas bloquer sur la lecture (file.read)
        dans le cas de l'utilisation de ssl
        """
        # read response from input file/socket, and parse it

        p, u = self.getparser()

        while 1:
            if sock:
                response = sock.recv(1024)
            else:
                response = file.read(1024)
            if not response:
                break
            if self.verbose:
                print "body:", repr(response)
            p.feed(response)
            if len(response) < 1024:
                break

        file.close()
        p.close()

        return u.close()
    
try:
    socket.setdefaulttimeout(15)
except:
    # non implémenté sur python 2.2
    pass

if registered:
    zephir = xmlrpclib.ServerProxy("https://zephir:zephir@"+config.adresse_zephir+":7080", transport=TransportEole())

def convert(objet):
    """Transforme les objets unicode contenus dans un objet en chaines
    """
    if type(objet) == list:
        l = []
        for item in objet:
            l.append(convert(item))
        return l
    if type(objet) == dict:
        dico={}
        for cle in objet.keys():
            dico[cle] = convert(objet[cle])
        return dico
    if type(objet) == unicode:
        string =  objet.encode(charset)
        return string
    return objet
        
def u(objet):
    if type(objet) == list:
        l = []
        for item in objet:
            l.append(u(item))
        return l
    if type(objet) == dict:
        dico={}
        for cle in objet.keys():
            dico[cle] = u(objet[cle])
        return dico
    if type(objet) == str:
        string = unicode(objet,charset)
        return string
    return objet

def log(action,etat,msg,date=''):
    """ fonction de log local et distant (si disponible) des actions zephir effectuées sur le serveur
    """
    # on insère le log actuel à la liste des logs à envoyer
    failed_logs = []
    ignored_logs = []
    if date == '':
        date = str(time.ctime())
    liste_logs = [[str(date),action,str(etat),msg]]
    # on regarde si il y a des logs différés (erreur de contact avec zephir lors d'un log précédent)
    try:
        fic_logs=open('%s/deffered_logs' % zephir_dir, 'r')
        # rqe : on limite à 20 le nombre d'anciens logs à remonter (pour éviter de bloquer trop longtemps)
        deffered_logs = fic_logs.read().strip().split('\n')
        if len(deffered_logs) > 20:
            old_logs = deffered_logs[:20]
            ignored_logs = deffered_logs[20:]
        else:
            old_logs = deffered_logs
        for log in old_logs:
            # on stocke ces logs dans la liste
            if log.count(';') > 0:
                liste_logs.append(log.split(';'))
        fic_logs.close()
    except:
        # le fichier n'existe pas
        # (tous les logs sont passés)
        pass

    erreur_zephir = 0
    # on essaie d'envoyer tous les logs via xmlrpc
    for log in liste_logs:
        if erreur_zephir == 0:
            # on n'essaie pas de réinsérer les logs suivants si il y a eu une erreur
            # (évite de répeter les timeouts de la connexion)
            try:
                date, action, etat, msg = log
                try:
                    msg = u(msg)
                except:
                    pass
                if log[0] != '':
                    res=zephir.uucp.log_serveur(config.id_serveur,u(date),u(action),int(etat),msg)
                else:
                    res=zephir.uucp.log_serveur(config.id_serveur,u(str(time.ctime())),u(date),int(etat),msg)
                if res[0] == 0:
                    # erreur d'insertion du log, mais zephir répond, on continue en stockant ce log
                    failed_logs.append(';'.join(log))
            except:
                # echec de la connexion à zephir
                # on stocke les informations de log dans le fichier deffered_logs
                # Ces logs seront parsés au prochain appel de cette fonction
                erreur_zephir = 1
                failed_logs.append(';'.join(log))
        else:
            failed_logs.append(';'.join(log))
            
    if ignored_logs != []:
        failed_logs.extend(ignored_logs)
    # on sauvegarde tous les logs non envoyés
    try:
        fic_logs=open('%s/deffered_logs' % zephir_dir, 'w')
        for log in failed_logs:
            fic_logs.write(log+'\n')
        fic_logs.close()
    except:
        # pas les droits pour écrire dans le fichier ?
        pass
        
    # Le log local est géré par les fonctions Zephir et Zecho
    return "ok"

# fonctions de verrouillage

lock_dir = '/var/lock'

def lock(tags):
    """fonction qui crée un ou plusieurs verrous"""
    retour = 1
    if os.path.isdir(lock_dir):
        if not type(tags) == list:
            tags=[tags]
        for tag in tags:
            try:
                fic_temp=open(lock_dir+os.sep+tag,"w").close()
            except:
                retour = 0
    else:
        retour = 0
    return retour

    
def unlock(tags):
    """fonction qui supprime un verrou"""
    try:
        if not type(tags) == list:
            tags=[tags]
        for tag in tags:
            if os.path.isfile(lock_dir+os.sep+tag):
                os.unlink(lock_dir+os.sep+tag)
    except:
        return 0
    return 1


def is_locked(tags):
    """fonction qui permet de savoir si un verrou particulier est posé"""
    if not type(tags) == list:
        tags=[tags]
    for tag in tags:
        if os.path.isfile(lock_dir+os.sep+tag):
            return 1
    else:
        return 0


def maj_client_zephir(zephir,adresse_zephir):
    """met à jour le client"""
    
    # on regarde si une mise à jour est nécessaire
    # récupération de la version locale
    out=os.popen('/usr/bin/dpkg -p zephir-client | grep Version')
    version_client = out.read().split(':')[1]
    out.close()
    version_client = "zephir-client_" + version_client.strip()
    res_maj = 2
    # vérification de la version présente sur zephir
    try:
        res=convert(zephir.maj_client(version_client))
    except Exception,e:
        # fonction de mise à jour non disponible sur cette version de zephir
        pass
    else:
        if res[0] == 0:
            print("pas de mise à jour disponible sur zephir")
        else:
            if res[1] != "OK":
                # une mise à jour est nécessaire
                rpm_client = res[1].encode(charset).strip()
                res_maj=os.system('%s/scripts/maj_client.sh %s %s' % (zephir_dir,adresse_zephir,rpm_client)) / 256
    return res_maj


def verif_transfer(zephir,archive):
    """vérifie la validité d'une archive recue et confirme sa réception à zephir"""
    verif_archive = convert(zephir.uucp.get_checksum(config.id_serveur,u(archive)))
    if verif_archive[0] == 0:
        log('ZEPHIR',1,'erreur de récupération du checksum pour '+archive+'.tar')
        return verif_archive
    else:
        md5string=xmlrpclib.base64.decodestring(verif_archive[1])
        toto=open(public_dir+os.sep+archive+'.md5','w')
        toto.write(md5string)
        toto.close()
        res = os.system('cd '+public_dir+' ; /usr/bin/md5sum -c '+toto.name)
        os.unlink(toto.name)
        if res == 0:
            # on informe zephir que le transfert s'est bien passé (rapport ?)
            res = convert(zephir.uucp.confirm_transfer(config.id_serveur,u(archive)))
            if res[0] == 1:
                return 1, 'ok'
            else:
                #erreur de supression de l'archive sur zephir ?
                return 1, res[1]
        else:
            # log('WARNING','checksum invalide pour '+archive+'.tar')
            return 0, 'checksum invalide pour '+archive+'.tar'

def mod_var_path(filepath):
    """change un chemin de fichier pour insérer 'variante' """
    filepath = list(os.path.split(filepath))
    filepath.insert(1,'variante')
    return os.sep.join(filepath)

def get_file_perms(base_dir, data_path, variante=False, filepath=""):
    """renvoie les informations de permissions associées à un fichier
    """
    data_dir = base_dir+os.sep+data_path
    # lecture des informations sur le fichier si disponibles
    if variante == True:
        permsfile = base_dir + os.sep + 'droits_variante'
    else:
        permsfile = base_dir+os.sep+'droits_zephir'
    if os.path.isfile(permsfile):
        f_rights = file(permsfile)
        data = f_rights.read().strip().split('\n')
        f_rights.close()
    else:
        data = []
    mode = user = group = ""
    recursive = False
    if filepath != "":
        if variante == True:
            result = {mod_var_path(filepath):[mode, user, group, recursive]}
        else:
            result = {filepath:[mode, user, group, recursive]}
    else:
        result = {}
    for line in data:
        if line.startswith(filepath) or filepath == "":
            if line.startswith(data_path):
                try:
                    filename, mode, user, group, recursive = line.split('#')
                    if recursive != '':
                        recursive = eval(recursive)
                    if variante == True:
                        result[mod_var_path(filename)] = [mode, user, group, recursive]
                    else:
                        result[filename] = [mode, user, group, recursive]
                    if filepath != "":
                        break
                except:
                    # fichier vide ?
                    pass
    return result


def register_prelude(zephir, ip_manager = None):
    """mise en place de la configuration prelude si nécessaire"""
    # recherche des managers disponibles
    from creole.config import eoledirs
    try:
        ret_code, managers = convert(zephir.prelude.get_managers())
    except xmlrpclib.ProtocolError:
        print("Vous n'êtes pas autorisé à modifier les fonctionnalités prelude")
        return False
    except:
        print "\n!! erreur de lecture de la configuration prelude sur zephir"
        return False
    if managers != {}:
        # choix du manager
        ip_conf = ''
        sondes = []
        try:
            # si une adresse_ip est définie, on la propose par défaut
            conf = EoleDict()
            conf.read_dir(eoledirs)
            conf.load_values('/etc/eole/config.eol')
            ip_conf = conf.get_value('adresse_ip_prelude_manager')
            sondes = conf.get_value('sondes_actives')
        except:
            pass
        saisie_default = ""
        choix = []
        print "\nserveurs preludemanager disponibles :\n"
        if ip_manager == None:
            id_default = managers.keys()[0]
            # si pas de manager spécifié, on propose un choix
            for id_man in managers.keys():
                ip_man = managers[id_man][0]
                libelle = managers[id_man][1]
                choix.append(id_man)
                if ip_man == ip_conf:
                    id_default = id_man
            saisie_default = "(%s) " % id_default
            id_man = "undef"
            while id_man not in choix:
                id_man = raw_input("\nIdentifiant du serveur preludemanager à utiliser %s? " % saisie_default)
                if id_man == "":
                    id_man = id_default
            ip_manager = managers[id_man][0]

        # mise en place de la configuration
        conf = EoleDict()
        conf.read('/etc/eole/dicos/prelude.xml')
        prelude_files = conf.files
        prelude_filelists = conf.filelists
        # instanciation des fichier de configuration prelude
        conf = EoleDict()
        # on récupère les autres variables
        conf.read_dir(eoledirs)
        conf.load_values('/etc/eole/config.eol')
        conf.set_value('adresse_ip_prelude_manager',ip_manager)
        if conf.save_values('/etc/eole/config.eol') == True:
            # et on ne garde que les fichiers prelude
            conf.files = prelude_files
            conf.filelists = prelude_filelists
            # instanciation des fichiers templates
            try:
                conf.instance()
            except:
                print '\n!! erreur de génération de la configuration prelude !!\n'
                return False
            # pour chaque sonde définie dans la configuration, on effectue un enregistrement (prelude-adduser)
            for sonde in sondes:
                # lancement du serveur d'enregistrement à distance
                pass_register = launch_adduser_server(zephir, id_man)
                if pass_register == "":
                    return False
                # lancer adduser avec les paramêtres donnés
                print " - enregistrement de la sonde %s" % sonde
                try:
                    options = conf.get_value('%s_options' % sonde)[0]
                except:
                    options = ""
                cmd_register = '/usr/bin/prelude-adduser register %s "idmef:w admin:r" %s %s --passwd=%s' % (sonde, ip_manager, options, pass_register)
                res = os.system(cmd_register)
                if res != 0:
                    print "\terreur lors de l'enregistrement de %s \n\t(%s)" % (sonde, cmd_register)
    return True



def launch_adduser_server(zephir, id_manager):
    # lancement de l'enregistrement avec prelude-adduser
    try:
        code, prelude_pass = convert(zephir.prelude.register_server(id_manager))
        assert code == 1
    except xmlrpclib.ProtocolError:
        print "\n!! Vous n'êtes pas autorisé à effectuer cette action !!\n"
        return ""
    except:
        print "\n!! erreur sur prelude manager !!\n"
        return ""
    return prelude_pass

