# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir listant le contenu de l'annaire LDAP
"""

from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import HTMLData, TableData
from zephir.monitor.agentmanager import status 
from twisted.python import log
import ldap,time

class LdapScribe(Agent):

    def __init__(self, name, **params):
        Agent.__init__(self, name, **params)
        """
        Gestion du status :
            par défaut : OK
            Erreur si pas de connexion
            Warning si erreur lors d'une ou plusieurs requêtes
        """
        self.status = status.OK()

        title1 = HTMLData("<h3>Nombre d'utilisateurs<h3>")
        self.table1 = TableData([
           ('name', 'Utilisateurs', {'align':'center'}, None),
           ('nb', 'Nombre' , {'align':'center'}, None)
        ])
        
        title2 = HTMLData("<h3>Nombre de groupes<h3>")
        self.table2 = TableData([
           ('name', 'Groupes', {'align':'center'}, None),
           ('nb', 'Nombre' , {'align':'center'}, None)
        ])
        
        title3 = HTMLData("<h3>Nombre de partages<h3>")
        self.table3 = TableData([
           ('name', 'Partages', {'align':'center'}, None),
           ('nb', 'Nombre' , {'align':'center'}, None)
        ])
        self.data = [title1, self.table1, title2, self.table2, title3, self.table3]
        
    def measure(self):
        try :
            conn=ldap.open('localhost')
        except :
            self.status = status.Error()
            return {'statistics1' : [], 'statistics2' : [], 'statistics3' : []}

        # reset à chaque mesure
        self.status = status.OK()
        self.fail = 0
        
        statistics1 = []
        nbeleves = self._requete(conn, "(&(objectclass=inetOrgPerson)(objectClass=Eleves))")
        statistics1.append (
            { 'name' : 'Eleves',
              'nb'   : nbeleves
            })
        nbprofs = self._requete(conn, "(&(objectclass=inetOrgPerson)(objectClass=administrateur))")
        statistics1.append (
            { 'name' : 'Professeurs',
              'nb'   : nbprofs
            })
        statistics1.append (
            { 'name' : self._color('Total'),
              'nb'   : self._color(self._add( (nbeleves, nbprofs) ))
            })

        statistics2 = []
        nbniv = self._requete(conn,"(&(objectclass=sambaGroupMapping)(description=Niveau *))")
        statistics2.append (
            { 'name' : 'Niveaux',
              'nb'   : nbniv
            })
        nbcl = self._requete(conn,"(&(objectclass=sambaGroupMapping)(description=Classe *))") 
        statistics2.append (
            { 'name' : 'Classes',
              'nb'   : nbcl
            })
        nbeq = self._requete(conn,"(&(objectclass=sambaGroupMapping)(description=Equipe *))")
        statistics2.append (
            { 'name' : 'Equipes pédagogiques',
              'nb'   : nbeq
            })
        nbmat = self._requete(conn,"(&(objectclass=sambaGroupMapping)(description=Matiere *))")
        statistics2.append (
            { 'name' : 'Matières',
              'nb'   : nbmat
            })
        nbtrav = self._requete(conn,"(&(objectclass=sambaGroupMapping)(|(description=Groupe *)(cn=eleves)(cn=professeurs)))")
        statistics2.append (
            { 'name' : 'Groupes de travail',
              'nb'   : nbtrav
            })
        nbspec = self._requete(conn,"(&(objectclass=sambaGroupMapping)(|(description=Administrateurs *)(cn=DomainComputers)(cn=DomainUsers)))")
        statistics2.append (
            { 'name' : 'Groupes spéciaux',
              'nb'   : nbspec
            })
        statistics2.append (
            { 'name' : self._color('Total'),
              'nb'   : self._color(self._add( (nbniv, nbcl, nbeq, nbmat, nbtrav, nbspec) ))
            })

        statistics3 = []
        statistics3.append (
            { 'name' : self._color('Total'),
              'nb'   : self._color(self._requete(conn,"(objectclass=sambaFileShare)"))
            })
       
        if self.fail >= 2 :
            self.status = status.Warn()
            
        return {'statistics1' : statistics1, 'statistics2' : statistics2, 'statistics3' : statistics3}


    def _requete(self, conn, filtre):
        time.sleep(0.05)
        try:
            return len(conn.search_s("o=gouv,c=fr", ldap.SCOPE_SUBTREE, filtre, ['']))
        except:
            try:
                # 2ème tentative
                time.sleep(0.1)
                return len(conn.search_s("o=gouv,c=fr", ldap.SCOPE_SUBTREE, filtre, ['']))
            except Exception, e:
                log.msg('erreur ldap : %s' % str(e))
                self.fail += 1
                return '???'

    def _color(self, string):
        return "<font color=\"red\">%s</font>" % string
    
    def _add (self, numbers) :
        # addition qui renvoit ??? si erreur
        total = 0
        for i in numbers:
            try :
                total += i
            except:
                return '???'
        return total

    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table2.table_data = self.last_measure.value['statistics2']
            self.table3.table_data = self.last_measure.value['statistics3']
            self.table1.table_data = self.last_measure.value['statistics1']

    def check_status(self):
        return self.status
    
