# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir pour afficher les partitions montées et leur utilisation
"""

from twisted.python import log
from twisted.internet import defer
from twisted.internet.utils import getProcessOutput

from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager import status
from zephir.monitor.agentmanager.data import HTMLData, TableData
from zephir.monitor.agentmanager.util import percent

# limites à partir des quelles on créé une alerte
SOFT_LIMIT = 90
HARD_LIMIT = 96

def stat_format(x):
    return "%.1f" % x

def perc2img(perc):
    p = int(perc[0:-1])
    if p >= HARD_LIMIT :
        color = 'red'
    elif p >= SOFT_LIMIT :
        color = 'orange'
    else:
        color = 'green'
        
    return """<img src="/static/img/%spix.gif" height="16" width="%s">
<img src="/static/img/whitepix.gif" height="16" width="%s">""" % (color,str(p),str(100-p))


class DiskSpace(Agent):

    def __init__(self, name,
                 **params):
        Agent.__init__(self, name, **params)
        self.table = TableData([
            ('name', 'Montage', {'align':'right'}, None),
            ('device', 'Partition', {'align':'left'}, None),
            ('type', 'Type', {'align':'right'}, None),
            ('perc', 'Utilisation', {'align':'right'}, None),
            ('used', 'Utilisé (Mo)', {'align':'right'}, None),
            ('free', 'Libre (Mo)', {'align':'right'}, None), 
            ('size', 'Taille (Mo)', {'align':'right'}, None),
            ('graph', 'Graphe', {'align':'left'}, None) ])
        self.data = [self.table]

    def measure(self):
        cmd = getProcessOutput('df',
                                   args = ['-kP','--print-type','--exclude-type=tmpfs','--exclude-type=usbfs'],
                                   env = {'LC_ALL': 'C'})
        cmd.addCallback(self.measure_process)
        return cmd
    
    def measure_process(self,mnt):
        # récupération des lignes
        lmnt = mnt.splitlines()
        # traitement mnt
        lmnt = lmnt[1:]
        liste=[]
        for lmn in lmnt :
            try:
                l = {}
                l['name'] = lmn.split()[6] # montage
                l['device'] = lmn.split()[0] # partition
                l['type'] = lmn.split()[1] # jointure sur le type
                l['perc'] = lmn.split()[5] # utilisation (%)
                l['free'] = int(lmn.split()[4])/1024 # dispo
                l['used'] = int(lmn.split()[3])/1024 # utilisé
                l['size'] = int(lmn.split()[2])/1024 # taille
                l['graph'] = perc2img(l['perc'])
            except:
                pass
            liste.append(l)
            self.measure_data[l['name']] = (int(l['perc'][:-1]), int(l['size']), int(l['used']), int(l['free']), l['type'])
 
        return {'statistics': liste}

    def check_status(self):
        err = 0
        warn = 0
        if self.last_measure is not None:
            # Alerte si une partition est remplie à plus de "LIMIT" (%)
            for device in self.last_measure.value['statistics']:
                # on ignore les cdroms pour les alertes
                if not device['type'] in ['iso9660','supermount']:
                    if int(device['perc'][:-1]) >= HARD_LIMIT :
                        err+=1
                    if int(device['perc'][:-1]) >= SOFT_LIMIT :
                        warn+=1
        if err == 1:
            return status.Error("%d partition remplie à plus de %d %%"%(err,HARD_LIMIT))
        if err > 1:
            return status.Error("%d partitions remplies à plus de %d %%"%(err,HARD_LIMIT))
        if warn == 1:
            return status.Warn("%d partition remplie à plus de %d %%"%(warn,SOFT_LIMIT))
        if warn > 1:
            return status.Warn("%d partitions remplies à plus de %d %%"%(warn,SOFT_LIMIT))
        return status.OK()
        
    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value['statistics']

