# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir pour les cartes réseau
"""

import os
from twisted.python import log
from twisted.internet import defer
from twisted.internet.utils import getProcessOutput, getProcessValue

from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import HTMLData, TableData
from zephir.monitor.agentmanager.util import status_to_img, boolean_to_onoff
from zephir.monitor.agentmanager import status

NETWORK_CHECK = '/usr/sbin/fping'

class Network(Agent):

    def __init__(self, name,
                 **params):
        Agent.__init__(self, name, **params)
        self.table = TableData([
            ('status', "état", {'align':'center'}, status_to_img), 
            ('interf', "Interface", {'align':'left'}, None),
            ])
        self.data = [self.table]


    def measure(self):
        self.mesures = []
        # on regarde si le vpn est activé
        cmd_rvp = "/usr/share/eole/test-rvp"
        if os.path.exists(cmd_rvp):
            return self.measure_rvp(cmd_rvp)
        else:
            return self.measure_network()

    def measure_rvp(self,cmd_rvp):
        # test vpn
        if not os.path.isfile('/var/lock/rvp'):
            status_rvp = 'Off'
            self.mesures.append({ 'interf': 'ipsec',
                                  'status': status_rvp })
            return self.measure_network()
        else:
            update = getProcessOutput(cmd_rvp, args=[])
            return update.addCallbacks(self.callback_rvp, self.errback_rvp)

    def callback_rvp(self, data):
        status_rvp = 'On'
        if 'alive' not in data:
            status_rvp = 'Off'
        self.mesures.append({ 'interf': 'ipsec',
                              'status': status_rvp })
        self.measure_data['ipsec'] = status_rvp
        # tests à effectuer avec tcpcheck
        return self.measure_network()

    def errback_rvp(self, err):
        self.mesures.append({ 'interf': 'ipsec',
                              'status': 'Off' })
        self.measure_data['ipsec'] = 'Off'
        # tests à effectuer avec tcpcheck
        return self.measure_network()

    def measure_network(self):
        # on détermine les interfaces à tester
        from creole.parsedico import parse_dico
        from ifconfig import ifconfig
        d = parse_dico()
        ic = ifconfig()
        # affiche seulement les interfaces ok (sans réslolution DNS)
        options = ['-a','-A','-q']
        self.interfaces = {}
        for var, value in d.items():
            if var.startswith('adresse_ip_eth') and value != "":
                self.interfaces[var[var.index('eth'):]] = ic.getStatus(var[var.index('eth'):])
            if var.startswith('adresse_ip_int') and value != "":
                self.interfaces['eth'+var[-1]] = ic.getStatus('eth'+var[-1])
        interfaces = self.interfaces.keys()
        interfaces.sort()
        for interf in interfaces:
            self.mesures.append({'interf': interf,
                                'status': boolean_to_onoff(self.interfaces[interf]) })
            self.measure_data[interf] = boolean_to_onoff(self.interfaces[interf])
        return {'networks': self.mesures}

    
    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value['networks']


    def check_status(self):
        """remonte une erreur si un des services est tombé"""
        if self.last_measure is not None:
            for service in self.last_measure.value['networks']:
                if service['status'] != 'On':
                    return status.Error()
        else:
            # pas de mesure connue
            return status.Unknown()
        return status.OK()

