# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Surveillance d'un onduleur géré par nut
Nécessite un test préliminaire normalement réalisé dans la conf
"""

from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import HTMLData, TableData
from zephir.monitor.agentmanager import status
from twisted.internet.utils import getProcessOutput

class Nut(Agent):
    def __init__(self, name, monitor, **params):
        Agent.__init__(self, name, **params)
        # monitor : nom de l'onduleur à surveiller (ex : ondul@localhost)
        self.monitor = monitor
        self.status = status.OK()
        self.table = TableData([
            ('label', '', {'align':'center'}, None),
            ('value', 'Valeur', {'align':'center'}, None)
            ])
        title = HTMLData("<h3>Surveillance de l'onduleur : <i>%s</i></h3>"%self.monitor)
        self.data = [title, self.table]
 
    def measure(self):
        self.status = status.OK()

        res = getProcessOutput('/bin/upsc',
                               args = [self.monitor],
                               env = {'LC_ALL': 'C'})
        res.addCallback(self.measure_process)
        res.addErrback(self.error)
        return res

    def measure_process(self, result):
        l = []
        for line in result.splitlines():
            data = line.split(':')
            if data[0].strip() == 'ups.status':
                ups_status = data[1].strip()
            if data[0].strip() == 'battery.charge':
                if float(data[1].strip()) > 100:
                    # bug du genre 750 au lieu de 75 !
                    data[1] = str(float(data[1])/10)
                ups_charge = data[1].strip()
            l.append({'label':data[0], 'value':data[1]})
        # ETAT (statut != "On Line")
        if 'OL' not in ups_status.split(' '):
            if float(ups_charge) < 20:
                self.status = status.Error("Charge batterie Faible")
            else:
                self.status = status.Warn("Fonctionnement sur batterie")
        return { 'statistics' : l}

    def error(self, result):
        self.status = status.Error()
        return { 'statistics' : [{'label':'Erreur', 'value':"""Echec de la connexion à l'onduleur"""}]}

    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value['statistics']
            
    def check_status(self):
        return self.status
 
