# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir pour les services distants
"""

import socket
from creole.parsedico import parse_dico
from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import TableData
from zephir.monitor.agentmanager import status
from zephir.monitor.agentmanager.util import status_to_img, boolean_to_onoff
from twisted.internet.utils import getProcessOutput
from os import system

def tcpcheck(mesures, label, address, port=80, timeout=3, fallback = None):
    """tentative de connexion sur un port donné d'un serveur distant"""
    tcp = getProcessOutput('/usr/bin/tcpcheck',
                                args = [str(timeout), address+':'+str(port)],
                                env = {'LC_ALL': 'C'})
    tcp.addCallback(process_tcp, mesures, label, fallback)
    return tcp
  
def process_tcp(tcp_result, mesures, label, fallback):
    # traitement du résultat de la commande
    if 'alive' in tcp_result:
        stat = "On"
    else:
        # si il y a une adresse de 'secours', on la teste aussi
        if fallback != None:
            return tcpcheck(mesures, 'Accès Distant', fallback)
        stat = "Off"
    mesures.append({'status':stat, 'service':label})
    return mesures

def resolve(address):
    """résolution d'adresse"""
    try:
        socket.gethostbyname(address)
    except socket.error:
        return False
    return True

def testweb(mesures, label, proxy, address1, address2):
    """
    test des site Web d'après le code de FonctionsEoleNg
    """    
    stat = "On"
    cmd_test = "http_proxy='%s' wget -q -t2 --connect-timeout=3 --delete-after %%s" % proxy
    code = system(cmd_test % address1)
    if code != 0:
        code = system(cmd_test % address2)
        if code != 0:
            stat = "Off"
    mesures.append({'status':stat, 'service':label})
    return mesures
    

class Web(Agent):

    def __init__(self, name, module,
                 **params):
        Agent.__init__(self, name, **params)
        self.module = module
        self.table = TableData([
            ('status', "État", {'align':'center'}, status_to_img), 
            ('service', "Service", {'align':'left'}, None),
            ])
        self.dns = 1
        self.data = [self.table]

    def measure(self):
        mesures = []
        # test dns distant
        conf_eole = parse_dico()
        try:
            tcpcheck(mesures, 'DNS distant 1', conf_eole['adresse_ip_dns_primaire'], 53)
            if conf_eole['adresse_ip_dns_secondaire'] != "":
                self.dns = 2
                tcpcheck(mesures, 'DNS distant 2', conf_eole['adresse_ip_dns_secondaire'], 53)
        except KeyError:
            # pas de variable dns_secondaire sur des fichiers de config non regénérés.
            pass
        ## test de la résolution ##
        stat = resolve('eole.orion.education.fr')
        mesures.append({'status':boolean_to_onoff(stat), 'service':'Résolution'})

        ## test du proxy ##
        proxy = ""
        http_proxy = ""
        if 'nom_cache_pere' in conf_eole and conf_eole['nom_cache_pere'] != '':     
            # Cache pere sur Amon
            proxy = conf_eole['nom_cache_pere']
            port  = conf_eole['port_cache_pere']
        elif 'proxy' in conf_eole and conf_eole['proxy'] == 'oui':
            # Proxy déclaré explicitement
            proxy = conf_eole['proxy_server']
            port  = conf_eole['proxy_port']
        if proxy != '':
            tcpcheck(mesures, 'Proxy (%s)' % proxy, proxy, port)
            http_proxy = "%s:%s" % (proxy, port)
        ## test accès distant ##
        return testweb(mesures, 'Accès Distant', http_proxy, 'eoleng.ac-dijon.fr', 'www.cru.fr')
    
    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value

    def check_status(self):
        """remonte une erreur si un des services est tombé"""
        if self.last_measure is not None:
            err_dns = 0
            for service in self.last_measure.value:
                self.measure_data[service['service']] = service['status']
                # cas du premier dns : on permet l'erreur
                if service['service'].startswith('DNS'):
                    if service['status'] != 'On':
                        err_dns += 1
                        # si tous les dns sont en panne : erreur
                        if err_dns == self.dns:
                            return status.Error()
                else:
                    if service['status'] != 'On':
                        return status.Error()
            if err_dns > 0:
                return status.Warn()
        else:
            # pas de mesure connue
            return status.Unknown()
        return status.OK()

