#!/usr/bin/env python
# -*- coding: utf-8 -*-
################################################################################
# Eole NG - 2008 
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# upgrade_distrib.py
#  
# script de mise à jour du serveur dans zephir après changement de distribution
# 
# - vérification du module actuel dans la base zephir
# - migration dans la base vers le module installé sur le serveur
#       
################################################################################

import sys, os, getpass, shutil, time, socket
registered = False
old_version = 2
new_version = 3
try:
    from zephir.eolerpclib import xmlrpclib
    from zephir.lib_zephir import *
    from zephir.zephir_conf import zephir_conf
    assert int(config.id_serveur) > 0
    registered = True
except:
    # serveur non enregistré sur zephir, on passe cette étape
    print("""Pas d'enregistrement dans une base Zephir détecté.""")


def upgrade():
    """migration du serveur dans zephir"""
    print "\nCette fonctionnalité nécessite un compte ayant la permission de migrer des serveurs \ndans l'application Zephir gérant ce serveur (%s)\n" % zephir_conf.adresse_zephir
    authentified, data = get_pwd(zephir_conf.adresse_zephir, 7080)
    if authentified == False:
        return data
    else:
        proxy = data
    id_new_mod = 0
    old_module = ""
    # récupération du module installé sur la machine
    module_local = file('/etc/eole/version').read().strip()
    # on récupère les informations sur le serveur dans la base zephir
    try:
        info_serveur = convert(proxy.serveurs.get_serveur(config.id_serveur))
    except xmlrpclib.ProtocolError:
        return("""erreur de permissions ou zephir non disponible""")
    if info_serveur[0] == 0:
        return("erreur de recherche du serveur dans la base zephir : "+str(info_serveur[1]))
    # on vérifie le n° de module du serveur dans la base zephir
    id_module = info_serveur[1][0]['module_actuel']
    modules = convert(proxy.modules.get_module())
    if modules[0] == 0:
        return("erreur de recherche du module dans la base zephir : "+str(modules[1]))
    # on détermine le numéro dans zephir du module installé
    for mod in modules[1]:
        if mod['libelle'] == module_local:
            id_new_mod = mod['id']
        if mod['id'] == id_module:
            old_module = mod['libelle']
    try:
        assert id_new_mod != 0 and old_module != ""
        print "\n** migration du module %s (%s) vers %s (%s) \n" % (old_module, id_module, module_local, id_new_mod)
        code, res = convert(proxy.serveurs.migrate_serveur(config.id_serveur, {}, id_new_mod))
        if code == 0:
            return ("erreur lors de la migration dans l'application : %s" % res)
    except xmlrpclib.ProtocolError:
        return("""erreur de permissions ou zephir non disponible""")
    except:
        return("""erreur lors de la migration dans l'application zephir""")
    # le migration s'est bien passée
    return ""

def upgrade_modules(version_src, version_dst):
    """fonction de copie des variantes dans le cas d'un upgrade du serveur zephir
    """
    from zephir import config
    from zephir.web.html.erreur import u, convert

    authentified, data = get_pwd(config.ADRESSE_ZEPHIR, config.PORT_ZEPHIR)
    if authentified == False:
        return False, data
    else:
        proxy = data

    # récupération des variantes sources (2.0)
    print "recherche des variantes sources existantes"
    res, modules_actuel = convert(proxy.modules.get_module())
    if res == 0:
        sys.exit('erreur de lecture des modules')
    mods = {}
    variantes = {}
    variantes_dst = {}

    res, variantes_ori = convert(proxy.modules.get_variante())
    if res == 0:
        sys.exit('erreur de lecture des variantes')

    for module in modules_actuel:
        tag = ''
        mod_name = module['libelle'][:module['libelle'].rindex('-')]
        if module['version'] == version_src:
            tag = 'src'
            # recherche des variantes existantes
            for var in variantes_ori:
                if var['module'] == module['id']:
                    if var['libelle'] != 'standard':
                        if variantes.has_key(module['id']):
                            variantes[module['id']].append((var['id'],var['libelle'],var['owner']))
                        else:
                            variantes[module['id']] = [(var['id'],var['libelle'],var['owner'])]

        elif module['version'] == version_dst:
            mod_name = module['libelle']
            mod_name = mod_name[:mod_name.rindex('-')]
            tag = 'dst'
            # recherche déjà migrées
            for var in variantes_ori:
                if var['module'] == module['id']:
                    if var['libelle'] != 'standard':
                        if variantes_dst.has_key(module['id']):
                            variantes_dst[module['id']].append(var['libelle'])
                        else:
                            variantes_dst[module['id']] = [var['libelle']]
        # stockage de la correspondance des modules source/dest
        if tag != '':
            if mod_name not in mods:
                mods[mod_name] = {tag:module['id']}
            else:
                mods[mod_name][tag] = module['id']

    # transfert des données des modules 2.0
    for modname, data in mods.items():
        if data.has_key('src') and data.has_key('dst'):
            # fichiers des valeurs par défaut du module
            module_eol_src = os.path.join(os.path.abspath(config.PATH_ZEPHIR),'modules',str(data['src']),'module.eol')
            if os.path.isfile(module_eol_src):
                module_eol_dst = os.path.join(os.path.abspath(config.PATH_ZEPHIR),'modules',str(data['dst']),'module.eol')
                print "reprise des valeurs par défaut pour le modules %s" % modname
                shutil.copy(module_eol_src, module_eol_dst)
            if data['src'] in variantes:
                for var in variantes[data['src']]:
                    id_var, libelle, owner = var
                    copy_needed = True
                    if libelle in variantes_dst.get(data['dst'],[]):
                        # si la variante existe déjà dans le module destination, on ne fait rien
                        copy_needed = False
                    if copy_needed:
                        # libelle de la nouvelle variante
                        code, res = proxy.modules.copy_variante(data['dst'],id_var,u(libelle),"",True)
                        print "création de la variante pour %s : %s-%s" % (modname, id_var, libelle)
    return True, "ok"

def get_pwd(addr, port):
    """lecture d'un login/passwd pour l'application zephir
    """
    login_ok = 0
    user = "toto"
    while login_ok == 0 and user != "":
        user = raw_input("Entrez votre login zephir (rien pour sortir) : ")
        if user != "":
            passwd = getpass.getpass("Mot de passe zephir pour %s : " % user)
            # création du proxy avec zephir
            proxy = xmlrpclib.ServerProxy("https://%s:%s@%s:%s" % (user, passwd, addr, port),transport=TransportEole())
            login_ok = 1
            try:
                res=convert(proxy.get_permissions(user))
            except xmlrpclib.ProtocolError:
                login_ok = 0
                print "\n Erreur d'authentification \n"
        else:
            return False, "! abandon de la procédure !"
    return True, proxy
    

if __name__ == '__main__':
    from creole.fonctionseole import get_module_name
    module = get_module_name()
    if registered:
        if module != "zephir":
            os.system('/etc/init.d/z_stats stop >/dev/null')
        else:
            os.system('/etc/init.d/zephir stop >/dev/null')
        # on bloque les fonctions zephir pendant la procédure
        lock(['uucp','maj','reconfigure','configure'])
        # upgrade des infos enregistrées sur zephir pour ce client
        res = upgrade()
        if res != "":
            print res
        # sauvegarde de la configuration locale sur zephir en fin de procédure
        unlock(['uucp','maj','reconfigure','configure'])
        if module != "zephir":
            os.system('/etc/init.d/z_stats start >/dev/null')
        else:
            os.system('/etc/init.d/zephir start >/dev/null')
        if res != "":
            sys.exit(1)
        print ('\nSauvegarde de la configuration actuelle sur Zephir')
        res = os.system('/usr/share/zephir/scripts/zephir_client save_files >/dev/null')
    # cas d'un upgrade sur zephir : mise à niveau des modules
    if get_module_name() ==  "zephir":
        # redémarrage de postgresql et zephir
        os.system('/etc/init.d/postgresql* restart')
        os.system('/etc/init.d/zephir restart')
        # attente du démarrage du service
        service_ok = False
        retries = 0
        print "\nMise en place des variantes sur les nouveaux modules (application zephir locale)"
        print "\nCette fonctionnalité nécessite un compte ayant la permission de créer des variantes"
        while not service_ok:
            try:
                # migration des variantes de la version précédente
                code, result = upgrade_modules(old_version, new_version)
                if not code:
                    print "\nErreur : %s\n" % result
                service_ok = True
            except:
                if retries == 0:
                    print "échec, attente du démarrage du service ..."
                if retries <= 5:
                    retries += 1
                    time.sleep(4)
                else:
                    print "Erreur, le service zephir ne répond pas"
                    service_ok = True
