#!/usr/bin/env python
# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# zephir_client.py
#  
# script principal de lancement d'actions par zephir
#       
###########################################################################

import time,os,sys,shutil
from zephir.eolerpclib import xmlrpclib
try:
    if not '/usr/share/eole/' in os.environ['PYTHONPATH'].split(':'):
        os.environ['PYTHONPATH'] = os.environ['PYTHONPATH']+':/usr/share/eole'
except KeyError:
    os.environ['PYTHONPATH'] = '/usr/share/eole'
try:
    import zephir.zephir_conf.zephir_conf as config
    from zephir.lib_zephir import *
except:
    print("\n** Ce serveur n'est pas enregistré sur Zephir **\n")
    sys.exit("Utilisez la procédure enregistrement_zephir\n")

# répertoire publique d'uucp
public_dir = '/var/spool/uucppublic'
        
def reconfigure(*args):
    os.system("sudo %s/scripts/reconfigure.zephir" % zephir_dir)

def configure(*args):
    os.system("sudo %s/scripts/configure.zephir" % zephir_dir)

def update_key(zephir, regen="", *args):
    if regen != "regen_certs":
        regen == ""
    os.system("sudo %s/scripts/update_key.zephir %s" % (zephir_dir, regen))

def reboot(*args):
    os.system("sudo %s/scripts/reboot.zephir" % zephir_dir)

def service_restart(zephir, service="", *args):
    os.system("sudo %s/scripts/service_restart.zephir %s" % (zephir_dir, service))

def maj_auto(zephir, time="", options="", *args):
    """fonction de mise à jour via zephir"""
    os.system("sudo %s/scripts/maj_auto.zephir %s %s" % (zephir_dir, time, options))

def maj_client(zephir):
    """fonction de mise à jour via zephir"""
    os.system("sudo %s/scripts/maj_client.zephir" % zephir_dir)

def del_lock(zephir,tag=['maj','configure','reconfigure','sauvegarde','uucp']):
    """ retire le fichier lock passé en paramètre
    et met à jour les logs zephir"""
    if is_locked(tag):
        # suppression du verrou
        unlock(tag)
        # log sur zephir
        log('LOCK',0,'supression des verrous dans /var/lock/subsys')
    else:
        log('LOCK',0,'verrous dans /var/lock/subsys déjà supprimés')
        
def call(zephir, *args):
    """fonction qui vide le répertoire temporaire et synchronise l'état du serveur par uucp"""
    # on interroge zephir pour savoir si les verrous doivent être annulés
    try:
        if convert(zephir.uucp.unlock(config.id_serveur))[1] == True:
            # suppression des verrous zephir si présents
            if is_locked(['maj','configure','reconfigure','sauvegarde','uucp']):
                del_lock(zephir)
            # on indique à zephir que les verrous sont supprimés
            convert(zephir.uucp.unlock(config.id_serveur,True))
    except:
        # erreur d'accès zephir ou fonction non implémentée
        pass
    # on vérifie qu'aucune tache n'a demandé un bloquage des fichiers
    if is_locked('sauvegarde'):
        print "verrou sur les tâches zephir détecté (/var/run/subsys/sauvegarde)"
        # cas où une sauvegarde est en cours, on ne fait rien (juste un log)
        log('LOCK',0,'taches zephir retardees : sauvegarde en cours')
    elif is_locked('uucp'):
        print "verrou sur les tâches zephir détecté (/var/run/subsys/uucp)"
        # cas d'une tâche non terminée, on saute un tour
        log('LOCK',1,'taches zephir retardees : verrou sur certaines taches zephir')
    else:
        # on commence par vérifier la liste des fonctions bloquées sur zephir
        try:
            locks = convert(zephir.serveurs.get_locks(config.id_serveur))
        except:
            # on ne bloque pas si cette phase échoue
            pass
        else:
            # mise en place du fichier de droits
            try:
                content = "\n".join([lock[0] for lock in locks[1]]) + "\n"
                file_lock = file(zephir_dir + "/zephir_locks","w")
                file_lock.write(content)
                file_lock.close()
            except:
                pass
            
        # on vide le spool uucp (évite l'envoi de plusieurs archives
        # de statistiques si il y a eu un pb de connexion avec zephir)
        try:
            shutil.rmtree("/var/spool/uucp/zephir")
        except:
            pass
        # FIXME loguer la réussite à chaque fois ??
        #log('LOCK',0,'aucun verrou dans /var/lock/subsys')
        # supression des fichiers et répertoires dans '/var/spool/uucppublic'
        fichiers = os.listdir(public_dir)
        for fic in fichiers:
            if os.path.isdir(public_dir+os.sep+fic):
                shutil.rmtree(public_dir+os.sep+fic)
            else:
                os.unlink(public_dir+os.sep+fic)
        # on met une date dans le rapport
        try:
            if not os.path.isfile("/tmp/rapport.zephir"):
                f=file("/tmp/rapport.zephir","w")
                f.close()
            os.system('/bin/chown uucp.uucp /tmp/rapport.zephir')
            # os.unlink('/tmp/rapport.zephir')
        except:
            pass
        # envoi par uucp de l'archive du site de diagnostic
    
        if not os.path.isfile('/tmp/site%s.tar' % config.id_serveur):
            log('SURVEILLANCE',1,'pas de données de surveillance (z_stats arrêté ?)')
            site_ok = 0
        else:
            site_ok = 1
            # signature md5 de l'archive
            cmd_md5 = """cd /tmp; /usr/bin/md5sum -b site%s.tar""" % config.id_serveur
            output = os.popen(cmd_md5)
            # lecture du résultat de la commande
            res_md5 = output.readlines()
            output.close()
            # envoi par uucp de l'archive du site de diagnostic
            cmd_uucp = """/usr/bin/uucp -r '/tmp/site%s.tar' 'zephir!~'""" % config.id_serveur
            res = os.system(cmd_uucp)

        # lancement de uucico pour l'envoi des fichiers
        # et la réception des commandes zephir
        print "\n   ** synchronisation Zephir en cours ** \n"
        try:
            # dans le cas de l'appel par crontab, on ajoute un délai
            # défini à l'enregistrement (aléatoire)
            if '--delay' in sys.argv:
                time.sleep(round(1 + random.random()*40,2))
        except:
            # pas de délai demandé
            pass
        # attente aléatoire pour éviter la surcharge des connexions sur zephir
        res=os.system('/usr/sbin/uucico -S zephir')
        if res != 0:
            print "erreur lors du transfert par uucp (uulog pour plus de détails)"
        else:
            print "   ** OK **\n"

        if site_ok:
            # vérification de l'archive et mise en place du site sur zephir
            try:
                # vérification de la version des agents
                new_agents = 1
                try:
                    from zephir.monitor.agentmanager import status
                except:
                    new_agents = 0
                res = convert(zephir.uucp.maj_site(config.id_serveur,res_md5,new_agents))
            except:
                # erreur de communication xmlrpc avec zephir
                # si on a déjà logué le pb, on ne le refait pas
                if os.path.isfile("/var/spool/uucp/no_contact.lock"):
                    pass
                else:
                    # on crée un fichier permettant de savoir qu'on a déjà logué la perte de contact
                    try:
                        f=file("/var/spool/uucp/no_contact.lock","w")
                        f.close()
                    except:
                        pass
                    log('SURVEILLANCE',1,'impossible de contacter le serveur zephir')
                try:
                    os.unlink('/tmp/site%s.tar' % config.id_serveur)
                except:
                    print "erreur de suppression de l'archive temporaire"
            else:
                if os.path.isfile("/var/spool/uucp/no_contact.lock"):
                    os.unlink("/var/spool/uucp/no_contact.lock")
                    log('SURVEILLANCE',0,'reprise du contact avec zephir')
                if res[0] != 1:
                    print 'erreur : '+res[1]
                    log('SURVEILLANCE',1,'erreur de mise en place des données de surveillance')
                else:
                    pass
                    # on ne logue pas la réussite
                    # log('SURVEILLANCE',0,'mise en place des données de surveillance terminée ')
                # on supprime l'archive une fois celle-ci envoyée
                try:
                    os.unlink('/tmp/site%s.tar' % config.id_serveur)
                except:
                    print "erreur de suppression de l'archive temporaire"


def lock_maj(zephir):
    res = os.system("sudo %s/scripts/lock-maj.zephir" % zephir_dir)
    # log('LOCK',1,'création du verrou /var/lock/subsys/sauvegarde')

def save_files(zephir):
    """lance la procédure d'envoi de la configuration du serveur sur zephir"""
    res = os.system('sudo %s/scripts/save_files.zephir >> /tmp/rapport.zephir 2>&1' % zephir_dir)
    if res == 0:
        return "ok"
    else:
        log('SAUVEGARDE',1,'erreur lors de la sauvegarde sur zephir')
        return "erreur save_files.zephir"
    

def main(exec_func=None):
    """ programme principal """
    if exec_func is None:
        try:
            exec_func=sys.argv[1]
            args = sys.argv[2:]
        except:
            sys.exit("donner la fonction à exécuter en argument")
    try:
        if args != []:
            exec_args = ",'%s'" % "','".join(args)
        else:
            exec_args = ''
        eval(exec_func+"(zephir%s)" % exec_args)
    except NameError, e:
        # permettre l'execution de scripts externes
        script_name="%s/scripts/" % zephir_dir + exec_func + '.zephir'
        # la fonction n'existe pas, on regarde si un script avec ce nom existe
        # dans /usr/share/zephir/scripts
        if os.path.isfile(script_name):
            os.system(script_name + " " + " ".join(args))
        else:
            print "fonction ou script inconnu : %s" % exec_func
        

if __name__ == '__main__':
    main()
